/*	
  Copyright(c) 2011 Christoph Schmalz <christoph.schmalz@informatik.uni-erlangen.de>
  
  This file is licensed under the terms of of the GNU General Public License, 
  version 3, as published by the Free Software Foundation. You can find it at:
  http://www.gnu.org/licenses/gpl.html
*/

#ifndef __PSPATTERN_H__
#define __PSPATTERN_H__

#include "opencv2/opencv.hpp"
using namespace cv;

///////////////////////////////////////////////////////////////////////////////
// Class: CPhaseShiftPattern
// Description:
///////////////////////////////////////////////////////////////////////////////

class CPhaseShiftPattern
{

// Public Functions:---------------------------------------
public:

	enum STRIPE_ORDER {
		STRIPE_ORDER_LEFT_RIGHT,
		STRIPE_ORDER_TOP_DOWN
	};

	// Constructor
	CPhaseShiftPattern( size_t iPatternWidth, size_t iPatternHeight,
						size_t iWaveLength, STRIPE_ORDER stripeOrder,
						double gamma=2.2) :
		m_iPatternWidth(iPatternWidth), m_iPatternHeight(iPatternHeight),
		m_iWaveLength(iWaveLength), m_stripeOrder(stripeOrder), m_dGamma(gamma) {}
	
	// Virtual destructor
	virtual ~CPhaseShiftPattern() {}

	void			DrawPhaseShiftPattern( 
						Mat&			img, 
						double			dPhaseOffset) const;	//rad!

	void			DrawGrayCode(	
						Mat&			image, 
						int				iSlideNr,
						int				iTotalSlides,
						bool			bInvert) const;

	bool			AreStripesHor() const 
						{return m_stripeOrder==STRIPE_ORDER_TOP_DOWN;}

	bool			IsValid() const
						{return m_iPatternWidth>0 && 
								m_iPatternHeight>0 && 
								m_iWaveLength>5; }

	float			GetNrOfGCStripes() const { return AreStripesHor() ?
						(2.f * m_iPatternHeight) / m_iWaveLength :
						(2.f * m_iPatternWidth) / m_iWaveLength; }

// Protected Attributes:--------------------------------------
protected:

	// The stripe order of the pattern
	STRIPE_ORDER		m_stripeOrder;

	// For phaseshift, graycode uses half of this as stripe width
	size_t				m_iWaveLength; 
	
	// The dimensions of the projection pattern 
	size_t				m_iPatternWidth;	// [pixel] 
	size_t				m_iPatternHeight;	// [pixel] 

	// Gamma correction for drawing
	double				m_dGamma;
};

#endif //__PSPATTERN_H__